@@ -1,4 +1,5 @@
 import pytest
+from _pytest.pytester import Testdir
 
 from anyio import get_all_backends
 
@@ -6,37 +7,41 @@ pytestmark = pytest.mark.filterwarnings(
     'ignore:The TerminalReporter.writer attribute is deprecated:pytest.PytestDeprecationWarning:')
 
 
-def test_plugin(testdir):
+def test_plugin(testdir: Testdir) -> None:
     testdir.makeconftest(
         """
-        import anyio
         import sniffio
         import pytest
 
+        from anyio import sleep
+
 
         @pytest.fixture
         async def async_fixture():
-            await anyio.sleep(0)
+            await sleep(0)
             return sniffio.current_async_library()
-        """
-    )
-
-    testdir.makepyfile(
-        """
-        import pytest
-
-        import sniffio
-        from anyio import get_all_backends, sleep
-        from hypothesis import strategies, given
 
 
         @pytest.fixture
         async def some_feature():
             yield None
             await sleep(0)
+        """
+    )
+
+    testdir.makepyfile(
+        """
+        import asyncio
+
+        import pytest
+        import sniffio
+        from hypothesis import strategies, given
+
+        from anyio import get_all_backends, sleep
+
 
         @pytest.mark.anyio
-        async def test_marked_test():
+        async def test_marked_test() -> None:
             # Test that tests marked with @pytest.mark.anyio are run
             pass
 
@@ -45,28 +50,10 @@ def test_plugin(testdir):
             # Test that async functions can use async fixtures
             assert async_fixture in get_all_backends()
 
-        @pytest.mark.parametrize('anyio_backend', ['curio'])
-        async def test_explicit_backend(anyio_backend):
-            # Test that when specifying the backend explicitly with parametrize, the correct
-            # backend is really used
-            assert anyio_backend == 'curio'
-            assert sniffio.current_async_library() == 'curio'
-
         def test_async_fixture_from_sync_test(anyio_backend_name, async_fixture):
             # Test that regular functions can use async fixtures too
             assert async_fixture == anyio_backend_name
 
-        @pytest.mark.parametrize('anyio_backend', ['asyncio'], scope='class')
-        class TestClassFixtures:
-            @pytest.fixture(scope='class')
-            async def async_class_fixture(self, anyio_backend):
-                await sleep(0)
-                return anyio_backend
-
-            def test_class_fixture_in_test_method(self, async_class_fixture, anyio_backend_name):
-                assert anyio_backend_name == 'asyncio'
-                assert async_class_fixture == 'asyncio'
-
         @pytest.mark.anyio
         async def test_skip_inline(some_feature):
             # Test for github #214
@@ -75,10 +62,83 @@ def test_plugin(testdir):
     )
 
     result = testdir.runpytest('-v')
-    result.assert_outcomes(passed=3 * len(get_all_backends()) + 2, skipped=len(get_all_backends()))
+    result.assert_outcomes(passed=3 * len(get_all_backends()), skipped=len(get_all_backends()))
 
 
-def test_autouse_async_fixture(testdir):
+def test_asyncio(testdir: Testdir) -> None:
+    testdir.makeconftest(
+        """
+        import asyncio
+        import pytest
+
+
+        @pytest.fixture(scope='class')
+        def anyio_backend():
+            return 'asyncio'
+
+        @pytest.fixture
+        async def setup_fail_fixture():
+            def callback():
+                raise RuntimeError('failing fixture setup')
+
+            asyncio.get_event_loop().call_soon(callback)
+            await asyncio.sleep(0)
+            yield None
+
+        @pytest.fixture
+        async def teardown_fail_fixture():
+            def callback():
+                raise RuntimeError('failing fixture teardown')
+
+            yield None
+            asyncio.get_event_loop().call_soon(callback)
+            await asyncio.sleep(0)
+        """
+    )
+
+    testdir.makepyfile(
+        """
+        import asyncio
+
+        import pytest
+
+        pytestmark = pytest.mark.anyio
+
+
+        class TestClassFixtures:
+            @pytest.fixture(scope='class')
+            async def async_class_fixture(self, anyio_backend):
+                await asyncio.sleep(0)
+                return anyio_backend
+
+            def test_class_fixture_in_test_method(self, async_class_fixture, anyio_backend_name):
+                assert anyio_backend_name == 'asyncio'
+                assert async_class_fixture == 'asyncio'
+
+        async def test_callback_exception_during_test() -> None:
+            def callback():
+                nonlocal started
+                started = True
+                raise Exception('foo')
+
+            started = False
+            asyncio.get_event_loop().call_soon(callback)
+            await asyncio.sleep(0)
+            assert started
+
+        async def test_callback_exception_during_setup(setup_fail_fixture):
+            pass
+
+        async def test_callback_exception_during_teardown(teardown_fail_fixture):
+            pass
+        """
+    )
+
+    result = testdir.runpytest('-v')
+    result.assert_outcomes(passed=2, failed=1, errors=2)
+
+
+def test_autouse_async_fixture(testdir: Testdir) -> None:
     testdir.makeconftest(
         """
         import pytest
@@ -111,11 +171,11 @@ def test_autouse_async_fixture(testdir):
         """
     )
 
-    result = testdir.runpytest('-v', '-p', 'no:curio')
+    result = testdir.runpytest('-v')
     result.assert_outcomes(passed=len(get_all_backends()))
 
 
-def test_cancel_scope_in_asyncgen_fixture(testdir):
+def test_cancel_scope_in_asyncgen_fixture(testdir: Testdir) -> None:
     testdir.makepyfile(
         """
         import pytest
@@ -126,7 +186,7 @@ def test_cancel_scope_in_asyncgen_fixture(testdir):
         @pytest.fixture
         async def asyncgen_fixture():
             async with create_task_group() as tg:
-                await tg.spawn(tg.cancel_scope.cancel)
+                tg.cancel_scope.cancel()
                 await sleep(1)
 
             yield 1
@@ -142,7 +202,7 @@ def test_cancel_scope_in_asyncgen_fixture(testdir):
     result.assert_outcomes(passed=len(get_all_backends()))
 
 
-def test_hypothesis_module_mark(testdir):
+def test_hypothesis_module_mark(testdir: Testdir) -> None:
     testdir.makepyfile(
         """
         import pytest
@@ -173,7 +233,7 @@ def test_hypothesis_module_mark(testdir):
     result.assert_outcomes(passed=len(get_all_backends()) + 1, xfailed=len(get_all_backends()))
 
 
-def test_hypothesis_function_mark(testdir):
+def test_hypothesis_function_mark(testdir: Testdir) -> None:
     testdir.makepyfile(
         """
         import pytest
