@@ -2,7 +2,9 @@ import asyncio
 import sys
 import threading
 import time
+from concurrent.futures import Future
 from functools import partial
+from typing import Any, List, NoReturn, Optional
 
 import pytest
 
@@ -18,18 +20,19 @@ else:
 pytestmark = pytest.mark.anyio
 
 
-async def test_run_in_thread_cancelled():
-    def thread_worker():
+async def test_run_in_thread_cancelled() -> None:
+    state = 0
+
+    def thread_worker() -> None:
         nonlocal state
         state = 2
 
-    async def worker():
+    async def worker() -> None:
         nonlocal state
         state = 1
         await to_thread.run_sync(thread_worker)
         state = 3
 
-    state = 0
     async with create_task_group() as tg:
         tg.start_soon(worker)
         tg.cancel_scope.cancel()
@@ -37,8 +40,8 @@ async def test_run_in_thread_cancelled():
     assert state == 1
 
 
-async def test_run_in_thread_exception():
-    def thread_worker():
+async def test_run_in_thread_exception() -> None:
+    def thread_worker() -> NoReturn:
         raise ValueError('foo')
 
     with pytest.raises(ValueError) as exc:
@@ -47,19 +50,20 @@ async def test_run_in_thread_exception():
     exc.match('^foo$')
 
 
-async def test_run_in_custom_limiter():
-    def thread_worker():
+async def test_run_in_custom_limiter() -> None:
+    num_active_threads = max_active_threads = 0
+
+    def thread_worker() -> None:
         nonlocal num_active_threads, max_active_threads
         num_active_threads += 1
         max_active_threads = max(num_active_threads, max_active_threads)
         event.wait(1)
         num_active_threads -= 1
 
-    async def task_worker():
+    async def task_worker() -> None:
         await to_thread.run_sync(thread_worker, limiter=limiter)
 
     event = threading.Event()
-    num_active_threads = max_active_threads = 0
     limiter = CapacityLimiter(3)
     async with create_task_group() as tg:
         for _ in range(4):
@@ -78,20 +82,22 @@ async def test_run_in_custom_limiter():
     (False, 'task'),
     (True, 'thread')
 ], ids=['uncancellable', 'cancellable'])
-async def test_cancel_worker_thread(cancellable, expected_last_active):
+async def test_cancel_worker_thread(cancellable: bool, expected_last_active: str) -> None:
     """
     Test that when a task running a worker thread is cancelled, the cancellation is not acted on
     until the thread finishes.
 
     """
-    def thread_worker():
+    last_active: Optional[str] = None
+
+    def thread_worker() -> None:
         nonlocal last_active
         from_thread.run_sync(sleep_event.set)
         time.sleep(0.2)
         last_active = 'thread'
         from_thread.run_sync(finish_event.set)
 
-    async def task_worker():
+    async def task_worker() -> None:
         nonlocal last_active
         try:
             await to_thread.run_sync(thread_worker, cancellable=cancellable)
@@ -100,7 +106,6 @@ async def test_cancel_worker_thread(cancellable, expected_last_active):
 
     sleep_event = Event()
     finish_event = Event()
-    last_active = None
     async with create_task_group() as tg:
         tg.start_soon(task_worker)
         await sleep_event.wait()
@@ -110,21 +115,39 @@ async def test_cancel_worker_thread(cancellable, expected_last_active):
     assert last_active == expected_last_active
 
 
+async def test_cancel_wait_on_thread() -> None:
+    event = threading.Event()
+    future: Future[bool] = Future()
+
+    def wait_event() -> None:
+        future.set_result(event.wait(1))
+
+    async with create_task_group() as tg:
+        tg.start_soon(partial(to_thread.run_sync, cancellable=True), wait_event)
+        await wait_all_tasks_blocked()
+        tg.cancel_scope.cancel()
+
+    await to_thread.run_sync(event.set)
+    assert future.result(1)
+
+
 @pytest.mark.parametrize('anyio_backend', ['asyncio'])
-async def test_asyncio_cancel_native_task():
-    async def run_in_thread():
+async def test_asyncio_cancel_native_task() -> None:
+    task: "Optional[asyncio.Task[None]]" = None
+
+    async def run_in_thread() -> None:
         nonlocal task
         task = current_task()
         await to_thread.run_sync(time.sleep, 0.2, cancellable=True)
 
-    task = None
     async with create_task_group() as tg:
         tg.start_soon(run_in_thread)
         await wait_all_tasks_blocked()
+        assert task is not None
         task.cancel()
 
 
-def test_asyncio_no_root_task(asyncio_event_loop):
+def test_asyncio_no_root_task(asyncio_event_loop: asyncio.AbstractEventLoop) -> None:
     """
     Regression test for #264.
 
@@ -133,7 +156,7 @@ def test_asyncio_no_root_task(asyncio_event_loop):
     that, uses the current task to set up a shutdown callback.
 
     """
-    async def run_in_thread():
+    async def run_in_thread() -> None:
         try:
             await to_thread.run_sync(time.sleep, 0)
         finally:
@@ -150,17 +173,17 @@ def test_asyncio_no_root_task(asyncio_event_loop):
             assert not t.is_alive()
 
 
-def test_asyncio_future_callback_partial(asyncio_event_loop):
+def test_asyncio_future_callback_partial(asyncio_event_loop: asyncio.AbstractEventLoop) -> None:
     """
     Regression test for #272.
 
     Ensures that futures with partial callbacks are handled correctly when the root task
     cannot be determined.
     """
-    def func(future):
+    def func(future: object) -> None:
         pass
 
-    async def sleep_sync():
+    async def sleep_sync() -> None:
         return await to_thread.run_sync(time.sleep, 0)
 
     task = asyncio_event_loop.create_task(sleep_sync())
@@ -168,12 +191,24 @@ def test_asyncio_future_callback_partial(asyncio_event_loop):
     asyncio_event_loop.run_until_complete(task)
 
 
-def test_asyncio_run_sync_no_asyncio_run(asyncio_event_loop):
+def test_asyncio_run_sync_no_asyncio_run(asyncio_event_loop: asyncio.AbstractEventLoop) -> None:
     """Test that the thread pool shutdown callback does not raise an exception."""
-    def exception_handler(loop, context=None):
+    def exception_handler(loop: object, context: Any = None) -> None:
         exceptions.append(context['exception'])
 
-    exceptions = []
+    exceptions: List[BaseException] = []
     asyncio_event_loop.set_exception_handler(exception_handler)
     asyncio_event_loop.run_until_complete(to_thread.run_sync(time.sleep, 0))
     assert not exceptions
+
+
+def test_asyncio_run_sync_multiple(asyncio_event_loop: asyncio.AbstractEventLoop) -> None:
+    """Regression test for #304."""
+    asyncio_event_loop.call_later(0.5, asyncio_event_loop.stop)
+    for _ in range(3):
+        asyncio_event_loop.run_until_complete(to_thread.run_sync(time.sleep, 0))
+
+    for t in threading.enumerate():
+        if t.name == 'AnyIO worker thread':
+            t.join(2)
+            assert not t.is_alive()
